{% if theme_navbar_footer_text %}{% set theme_extra_navbar=theme_navbar_footer_text %}{% endif %} <!-- To handle the deprecated key -->
{% if theme_extra_navbar %}
<div class="navbar_extra_footer">
  {{ theme_extra_navbar }}
</div>
{% endif %}
