
<nav class="bd-links" id="bd-docs-nav" aria-label="Main">
  <div class="bd-toc-item active">
    {{ sbt_generate_nav_html(include_item_names=True,
    with_home_page=theme_home_page_in_toc, show_depth=theme_show_navbar_depth)
    }}
  </div>
</nav>

{% include "lite-demo.html" %}
